;  Copyright (c) 2019, Vintage Computing Carinthia
;  Author: WOZ    -    http://www.pin4.at/    -    woz <at> pin4 <dot> at  
;  All rights reserved.
;
;  Petscii decompressor, unpacks directly to the screen
;
;  Redistribution and use in source and binary forms, with or without
;  modification, are permitted provided that the following conditions are met:
;      * Redistributions of source code must retain the above copyright
;        notice, this list of conditions and the following disclaimer.
;      * Redistributions in binary form must reproduce the above copyright
;        notice, this list of conditions and the following disclaimer in the
;        documentation and/or other materials provided with the distribution.
;      * Neither the name of the author(s) nor the
;        names of its contributors may be used to endorse or promote products
;        derived from this software without specific prior written permission.
;      * Commercial use prohibited without permission of the author(s).
;
;  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
;  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
;  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
;  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY
;  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
;  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
;  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
;  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
;  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
;  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;

;===============================================
; BUILD WITH
; cl65 -o decompressor.prg -t c64 -C c64-asm.cfg -l decompressor.asm  decompressor.s
;===============================================

;screen addresses (you may change sptr+1/sptr+2 directly for other screen locations)
screen=$0400
colour=$D800

;zeropage pointer
inv = $f9
col = $fa
dbl1a = $fb
dbl1b = $fc
dbl2a = $fd
dbl2b = $fe
cnt = $ff

;===============================================
;BASIC start header

.org $0801

.byte $0B, $08, $20, $03, $9E, $32, $30, $36, $31, $00, $00, $00   ; SYS 2061

;===============================================
;ASM start

;-----------------------------------------------
; set some variables we will neeed
_decompress:
  lda #$00
  sta inv        ; clear inverse flag
  sta col        ; clear color
  sta cnt        ; clear counter
  lda _example
  sta $D020
  lda _example+1
  sta $D021
  lda _example+5
  beq mainloop    ; have no double definitions

;-----------------------------------------------
; set double data definitions
  sta dbl1a
  lda _example+6
  sta dbl1b
  lda _example+7
  sta dbl2a
  lda _example+8
  sta dbl2b
  ; for now we assume there is not carry to the upper byte involved
  inc mainloop+1
  inc mainloop+1
  inc mainloop+1

;-----------------------------------------------
; here we run through the data until we reach 0x00
mainloop:
  lda _example+6
  beq exit
  inc mainloop+1
  bne next0
  inc mainloop+2
next0:
  cmp #$60
  bcs unroll
  ; char < $60, set repetition for next data
  sta cnt
  jmp mainloop

; we unroll here runlength encoded data
unroll:
  tay
  jsr usebyte
  ; continue in stream cnt is zero
  lda cnt
  beq mainloop
  ; otherwise decrement cnt, get back char and do it again
  dec cnt
  tya
  jmp unroll

;-----------------------------------------------
; we decode a single data field
usebyte:
  cmp #$70
  beq invon
  cmp #$71
  beq invoff
  cmp #$78
  beq dbl1
  cmp #$79
  beq dbl2
  cmp #$80
  bcs write  ; >=$80, poke character
setcol:
  and #$0F   ; what remains here must be colour
  sta col
  rts
invon:
  lda #$80   ; set inversion
  sta inv
  rts
invoff:
  lda #$00   ; set normal
  sta inv
  rts
dbl1:
  lda dbl1a
  jsr usebyte
  lda dbl1b
  jmp usebyte
dbl2:
  lda dbl2a
  jsr usebyte
  lda dbl2b
  jmp usebyte
write:
  and #$7F  ; character or inv. character
  ora inv
sptr:
  sta screen
  inc sptr+1
  bne next1
  inc sptr+2
next1:
  lda col
cptr:
  sta colour
  inc cptr+1
  bne next2
  inc cptr+2
next2:
  rts

;-----------------------------------------------
; finally, we are done (we wait for a keypress)
exit:
  lda $c6
  beq exit
  rts

;===============================================
;example picture 1, author LgK
; runs in 92364 cycles  -->  ~100ms
_example:
; compressed petscii sequence data
.byte $00, $00, $28, $19, $00, $00, $5f, $a0, $5f, $a0, $16, $a0, $67, $c9, $d5, $c9, $a0, $d5, $c3, $c9, $a0, $d5, $c3, $c9, $1c, $a0, $c2, $ca, $db, $a0, $dd, $02
.byte $a0, $dd, $1e, $a0, $c2, $a0, $dd, $a0, $dd, $02, $a0, $dd, $1e, $a0, $ca, $c0, $fd, $a0, $ca, $c3, $cb, $a0, $ca, $c3, $cb, $5f, $a0, $08, $a0, $c9, $d5, $25
.byte $a0, $c2, $dd, $25, $a0, $ca, $fd, $a0, $89, $a0, $8e, $a0, $94, $a0, $81, $a0, $87, $a0, $85, $1a, $a0, $d5, $cb, $25, $a0, $dd, $26, $a0, $ca, $cb, $a0, $8f
.byte $a0, $8d, $a0, $90, $a0, $95, $a0, $94, $a0, $89, $a0, $8e, $a0, $87, $16, $a0, $d5, $cb, $25, $a0, $dd, $26, $a0, $ca, $cb, $a0, $81, $a0, $92, $a0, $89, $a0
.byte $8e, $a0, $94, $a0, $88, $a0, $89, $a0, $81, $5f, $a0, $5f, $a0, $10, $a0, $00

; see http://6502org.wikidot.com/software-incdec for some coding techniques used
