;  Copyright (c) 2019, Vintage Computing Carinthia
;  Author: WOZ    -    http://www.pin4.at/    -    woz <at> pin4 <dot> at  
;  All rights reserved.
;
;  Petscii SEQ reader, unpacks directly to the screen
;
;  Redistribution and use in source and binary forms, with or without
;  modification, are permitted provided that the following conditions are met:
;      * Redistributions of source code must retain the above copyright
;        notice, this list of conditions and the following disclaimer.
;      * Redistributions in binary form must reproduce the above copyright
;        notice, this list of conditions and the following disclaimer in the
;        documentation and/or other materials provided with the distribution.
;      * Neither the name of the author(s) nor the
;        names of its contributors may be used to endorse or promote products
;        derived from this software without specific prior written permission.
;      * Commercial use prohibited without permission of the author(s).
;
;  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
;  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
;  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
;  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY
;  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
;  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
;  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
;  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
;  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
;  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;

;===============================================
; BUILD WITH
; cl65 -o seqreader.prg -t c64 -C c64-asm.cfg -l seqreader.asm  seqreader.s
;===============================================

;screen addresses (you may change sptr+1/sptr+2 directly for other screen locations)
screen=$0400
colour=$D800

;zeropage pointer
inv = $fb
col = $fc
p1 = $fd
p2 = $fe

;===============================================
;BASIC start header

.org $0801

.byte $0B, $08, $20, $03, $9E, $32, $30, $36, $31, $00, $00, $00   ; SYS 2061

;===============================================
;ASM start

;-----------------------------------------------
_decoder:
  lda #$00
  sta $D020
  lda #$00
  sta $D021
  lda #$00
  sta inv
  sta col
loop:
  lda _example
  inc loop+1
  bne n0
  inc loop+2
n0:
  cmp #$90  ; black
  bne n1
  lda #0
  sta col
  jmp loop
n1:
  cmp #$05  ; white
  bne n2
  lda #1
  sta col
  jmp loop
n2:
  cmp #$1C  ; red
  bne n3
  lda #2
  sta col
  jmp loop
n3:
  cmp #$9F  ; cyan
  bne n4
  lda #3
  sta col
  jmp loop
n4:
  cmp #$9C  ; pink
  bne n5
  lda #4
  sta col
  jmp loop
n5:
  cmp #$1E  ; green
  bne n6
  lda #5
  sta col
  jmp loop
n6:
  cmp #$1F  ; blue
  bne n7
  lda #6
  sta col
  jmp loop
n7:
  cmp #$9E  ; yellow
  bne n8
  lda #7
  sta col
  jmp loop
n8:
  cmp #$81  ; orange
  bne n9
  lda #8
  sta col
  jmp loop
n9:
  cmp #$95  ; brown
  bne n10
  lda #9
  sta col
  jmp loop
n10:
  cmp #$96  ; light red
  bne n11
  lda #10
  sta col
  jmp loop
n11:
  cmp #$97  ; dark gray
  bne n12
  lda #11
  sta col
  jmp loop
n12:
  cmp #$98  ; medium gray
  bne n13
  lda #12
  sta col
  jmp loop
n13:
  cmp #$99  ; light green
  bne n14
  lda #13
  sta col
  jmp loop
n14:
  cmp #$9A  ; light blue
  bne n15
  lda #14
  sta col
  jmp loop
n15:
  cmp #$9B  ; light gray
  bne n16
  lda #15
  sta col
  jmp loop
n16:
  cmp #$12  ; inversion on
  bne n17
  lda #$80
  sta inv
  jmp loop
n17:
  cmp #$92  ; inversion on
  bne n18
  lda #0
  sta inv
  jmp loop
n18:
  tay
  and #$e0
  cmp #$20
  bne n19
  tya
  jmp print
n19:
  cmp #$40
  bne n20
  tya
  and #$1f
  jmp print
n20:
  cmp #$60
  beq n21
  cmp #$c0
  bne n22
n21:
  tya
  and #$1f
  ora #$40
  jmp print
n22:
  cmp #$A0
  beq n23
  cmp #$E0
  bne n24
n23:
  tya
  and #$1f
  ora #$60
  jmp print
n24:
  jmp loop

print:
  and #$7f
  ora inv
sptr:
  sta screen
  inc sptr+1
  bne next1
  inc sptr+2
next1:
  lda col
cptr:
  sta colour
  inc cptr+1
  bne next2
  inc cptr+2
  lda #(>colour)+4
  cmp cptr+2
  beq exit
next2:
  jmp loop

;-----------------------------------------------
; finally, we are done (we wait for a keypress)
exit:
  lda $c6
  beq exit
  rts

;===============================================
;example picture 1, author LgK
; runs in 170854 cycles  -->  ~180ms
_example:
; petscii sequence data
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $9e, $69, $75, $69, $20, $75, $63, $69, $20
.byte $75, $63, $69, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $62, $6a, $7b, $20, $7d, $20, $20, $20, $7d, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $62, $20, $7d, $20, $7d, $20, $20, $20, $7d, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $6a, $60, $bd, $20, $6a, $63, $6b, $20, $6a, $63, $6b, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $69, $75, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $62, $7d, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $6a, $bd, $20, $49, $20, $4e, $20, $54, $20, $41, $20, $47
.byte $20, $45, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $75, $6b, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $7d, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $6a, $6b, $20, $4f, $20, $4d, $20, $50, $20, $55, $20, $54, $20, $49, $20, $4e, $20, $47, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $75, $6b, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $7d, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $6a, $6b, $20, $41, $20, $52, $20, $49, $20, $4e, $20, $54, $20, $48, $20, $49, $20, $41, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $20
.byte $20, $20, $20, $20, $20, $20, $20, $20, $20

; see http://6502org.wikidot.com/software-incdec for some coding techniques used
